@@ -31,17 +31,20 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <gst/gst.h>
 #include <gst/gstvalue.h>
 #include <gst/video/video.h>
 #include <gst/riff/riff-ids.h>
-#include <cogl/cogl.h>
 #include <string.h>
 
+/* We just need the public Cogl api for cogl-gst but we first need to
+ * undef COGL_COMPILATION to avoid getting an error that normally
+ * checks cogl.h isn't used internally. */
+#undef COGL_COMPILATION
+#include <cogl/cogl.h>
+
 #include "cogl-gst-video-sink.h"
 
 #define COGL_GST_DEFAULT_PRIORITY G_PRIORITY_HIGH_IDLE
@@ -198,7 +201,7 @@ cogl_gst_video_sink_attach_frame (CoglGstVideoSink *sink,
   for (i = 0; i < G_N_ELEMENTS (priv->frame); i++)
     if (priv->frame[i] != NULL)
       cogl_pipeline_set_layer_texture (pln, i + priv->custom_start,
-                                       COGL_TEXTURE (priv->frame[i]));
+                                       priv->frame[i]);
 }
 
 static CoglBool
@@ -467,10 +470,8 @@ video_texture_new_from_data (CoglContext *ctx,
                              int width,
                              int height,
                              CoglPixelFormat format,
-                             CoglPixelFormat internal_format,
                              int rowstride,
-                             const uint8_t *data,
-                             CoglError **error)
+                             const uint8_t *data)
 {
   CoglBitmap *bitmap;
   CoglTexture *tex;
@@ -486,9 +487,7 @@ video_texture_new_from_data (CoglContext *ctx,
        is_pot (cogl_bitmap_get_height (bitmap))) ||
       cogl_has_feature (ctx, COGL_FEATURE_ID_TEXTURE_NPOT_BASIC))
     {
-      tex = COGL_TEXTURE (cogl_texture_2d_new_from_bitmap (bitmap,
-                                                           internal_format,
-                                                           &internal_error));
+      tex = cogl_texture_2d_new_from_bitmap (bitmap);
       if (!tex)
         {
           cogl_error_free (internal_error);
@@ -501,12 +500,8 @@ video_texture_new_from_data (CoglContext *ctx,
   if (!tex)
     {
       /* Otherwise create a sliced texture */
-      CoglTexture2DSliced *tex_2ds =
-        cogl_texture_2d_sliced_new_from_bitmap (bitmap,
-                                                -1, /* no maximum waste */
-                                                internal_format,
-                                                error);
-      tex = COGL_TEXTURE (tex_2ds);
+      tex = cogl_texture_2d_sliced_new_from_bitmap (bitmap,
+                                                    -1); /* no maximum waste */
     }
 
   cogl_object_unref (bitmap);
@@ -534,9 +529,9 @@ cogl_gst_rgb24_upload (CoglGstVideoSink *sink,
 
   priv->frame[0] = video_texture_new_from_data (priv->ctx, priv->info.width,
                                                 priv->info.height,
-                                                format, format,
+                                                format,
                                                 priv->info.stride[0],
-                                                frame.data[0], NULL);
+                                                frame.data[0]);
 
   gst_video_frame_unmap (&frame);
 
@@ -580,9 +575,9 @@ cogl_gst_rgb32_upload (CoglGstVideoSink *sink,
 
   priv->frame[0] = video_texture_new_from_data (priv->ctx, priv->info.width,
                                                 priv->info.height,
-                                                format, format,
+                                                format,
                                                 priv->info.stride[0],
-                                                frame.data[0], NULL);
+                                                frame.data[0]);
 
   gst_video_frame_unmap (&frame);
 
@@ -623,22 +618,22 @@ cogl_gst_yv12_upload (CoglGstVideoSink *sink,
     video_texture_new_from_data (priv->ctx,
                                  GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 0),
                                  GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 0),
-                                 format, format,
-                                 priv->info.stride[0], frame.data[0], NULL);
+                                 format,
+                                 priv->info.stride[0], frame.data[0]);
 
   priv->frame[2] =
     video_texture_new_from_data (priv->ctx,
                                  GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 1),
                                  GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 1),
-                                 format, format,
-                                 priv->info.stride[1], frame.data[1], NULL);
+                                 format,
+                                 priv->info.stride[1], frame.data[1]);
 
   priv->frame[1] =
     video_texture_new_from_data (priv->ctx,
                                  GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 2),
                                  GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 2),
-                                 format, format,
-                                 priv->info.stride[2], frame.data[2], NULL);
+                                 format,
+                                 priv->info.stride[2], frame.data[2]);
 
   gst_video_frame_unmap (&frame);
 
@@ -668,22 +663,22 @@ cogl_gst_i420_upload (CoglGstVideoSink *sink,
     video_texture_new_from_data (priv->ctx,
                                  GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 0),
                                  GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 0),
-                                 format, format,
-                                 priv->info.stride[0], frame.data[0], NULL);
+                                 format,
+                                 priv->info.stride[0], frame.data[0]);
 
   priv->frame[1] =
     video_texture_new_from_data (priv->ctx,
                                  GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 1),
                                  GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 1),
-                                 format, format,
-                                 priv->info.stride[1], frame.data[1], NULL);
+                                 format,
+                                 priv->info.stride[1], frame.data[1]);
 
   priv->frame[2] =
     video_texture_new_from_data (priv->ctx,
                                  GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 2),
                                  GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 2),
-                                 format, format,
-                                 priv->info.stride[2], frame.data[2], NULL);
+                                 format,
+                                 priv->info.stride[2], frame.data[2]);
 
   gst_video_frame_unmap (&frame);
 
@@ -715,9 +710,9 @@ cogl_gst_yv12_glsl_setup_pipeline (CoglGstVideoSink *sink,
                          "cogl_gst_sample_video%i (vec2 UV)\n"
                          "{\n"
                          "  float y = 1.1640625 * "
-                         "(texture2D (cogl_sampler%i, UV).g - 0.0625);\n"
-                         "  float u = texture2D (cogl_sampler%i, UV).g - 0.5;\n"
-                         "  float v = texture2D (cogl_sampler%i, UV).g - 0.5;\n"
+                         "(texture2D (cogl_sampler%i, UV).a - 0.0625);\n"
+                         "  float u = texture2D (cogl_sampler%i, UV).a - 0.5;\n"
+                         "  float v = texture2D (cogl_sampler%i, UV).a - 0.5;\n"
                          "  vec4 color;\n"
                          "  color.r = y + 1.59765625 * v;\n"
                          "  color.g = y - 0.390625 * u - 0.8125 * v;\n"
@@ -811,9 +806,9 @@ cogl_gst_ayuv_upload (CoglGstVideoSink *sink,
 
   priv->frame[0] = video_texture_new_from_data (priv->ctx, priv->info.width,
                                                 priv->info.height,
-                                                format, format,
+                                                format,
                                                 priv->info.stride[0],
-                                                frame.data[0], NULL);
+                                                frame.data[0]);
 
   gst_video_frame_unmap (&frame);
 
