@@ -69,7 +69,6 @@ _cogl_onscreen_new (void)
   _cogl_framebuffer_init (COGL_FRAMEBUFFER (onscreen),
                           ctx,
                           COGL_FRAMEBUFFER_TYPE_ONSCREEN,
-                          COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                           0x1eadbeef, /* width */
                           0x1eadbeef); /* height */
   /* NB: make sure to pass positive width/height numbers here
@@ -104,7 +103,6 @@ cogl_onscreen_new (CoglContext *ctx, int width, int height)
   _cogl_framebuffer_init (COGL_FRAMEBUFFER (onscreen),
                           ctx,
                           COGL_FRAMEBUFFER_TYPE_ONSCREEN,
-                          COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                           width, /* width */
                           height); /* height */
 
@@ -633,8 +631,8 @@ _cogl_framebuffer_winsys_update_size (CoglFramebuffer *framebuffer,
 
   cogl_framebuffer_set_viewport (framebuffer, 0, 0, width, height);
 
-  if (!(framebuffer->context->private_feature_flags &
-        COGL_PRIVATE_FEATURE_DIRTY_EVENTS))
+  if (!_cogl_has_private_feature (framebuffer->context,
+                                  COGL_PRIVATE_FEATURE_DIRTY_EVENTS))
     _cogl_onscreen_queue_full_dirty (COGL_ONSCREEN (framebuffer));
 }
 
