@@ -27,50 +27,52 @@
 #include <cogl/cogl-pipeline.h>
 
 #include "cogl-context.h"
-
+#include "cogl-flags.h"
 
 COGL_BEGIN_DECLS
 
 typedef enum
 {
-  COGL_PRIVATE_FEATURE_TEXTURE_2D_FROM_EGL_IMAGE = 1L<<0,
-  COGL_PRIVATE_FEATURE_MESA_PACK_INVERT = 1L<<1,
-  COGL_PRIVATE_FEATURE_OFFSCREEN_BLIT = 1L<<2,
-  COGL_PRIVATE_FEATURE_FOUR_CLIP_PLANES = 1L<<3,
-  COGL_PRIVATE_FEATURE_PBOS = 1L<<4,
-  COGL_PRIVATE_FEATURE_VBOS = 1L<<5,
-  COGL_PRIVATE_FEATURE_EXT_PACKED_DEPTH_STENCIL = 1L<<6,
-  COGL_PRIVATE_FEATURE_OES_PACKED_DEPTH_STENCIL = 1L<<7,
-  COGL_PRIVATE_FEATURE_TEXTURE_FORMAT_BGRA8888 = 1L<<8,
-  COGL_PRIVATE_FEATURE_UNPACK_SUBIMAGE = 1L<<9,
-  COGL_PRIVATE_FEATURE_SAMPLER_OBJECTS = 1L<<10,
-  COGL_PRIVATE_FEATURE_READ_PIXELS_ANY_FORMAT = 1L<<11,
-  COGL_PRIVATE_FEATURE_ALPHA_TEST = 1L<<12,
-  COGL_PRIVATE_FEATURE_FORMAT_CONVERSION = 1L<<13,
-  COGL_PRIVATE_FEATURE_QUADS = 1L<<14,
-  COGL_PRIVATE_FEATURE_BLEND_CONSTANT = 1L<<15,
-  COGL_PRIVATE_FEATURE_QUERY_FRAMEBUFFER_BITS = 1L<<16,
-  COGL_PRIVATE_FEATURE_BUILTIN_POINT_SIZE_UNIFORM = 1L<<17,
-  COGL_PRIVATE_FEATURE_QUERY_TEXTURE_PARAMETERS = 1L<<18,
-  COGL_PRIVATE_FEATURE_ALPHA_TEXTURES = 1L<<19,
-  COGL_PRIVATE_FEATURE_TEXTURE_SWIZZLE = 1L<<20,
-  COGL_PRIVATE_FEATURE_TEXTURE_MAX_LEVEL = 1L<<21,
-  COGL_PRIVATE_FEATURE_ARBFP = 1L<<22,
-  COGL_PRIVATE_FEATURE_OES_EGL_SYNC = 1L<<23,
+  COGL_PRIVATE_FEATURE_TEXTURE_2D_FROM_EGL_IMAGE,
+  COGL_PRIVATE_FEATURE_MESA_PACK_INVERT,
+  COGL_PRIVATE_FEATURE_OFFSCREEN_BLIT,
+  COGL_PRIVATE_FEATURE_FOUR_CLIP_PLANES,
+  COGL_PRIVATE_FEATURE_PBOS,
+  COGL_PRIVATE_FEATURE_VBOS,
+  COGL_PRIVATE_FEATURE_EXT_PACKED_DEPTH_STENCIL,
+  COGL_PRIVATE_FEATURE_OES_PACKED_DEPTH_STENCIL,
+  COGL_PRIVATE_FEATURE_TEXTURE_FORMAT_BGRA8888,
+  COGL_PRIVATE_FEATURE_UNPACK_SUBIMAGE,
+  COGL_PRIVATE_FEATURE_SAMPLER_OBJECTS,
+  COGL_PRIVATE_FEATURE_READ_PIXELS_ANY_FORMAT,
+  COGL_PRIVATE_FEATURE_ALPHA_TEST,
+  COGL_PRIVATE_FEATURE_FORMAT_CONVERSION,
+  COGL_PRIVATE_FEATURE_QUADS,
+  COGL_PRIVATE_FEATURE_BLEND_CONSTANT,
+  COGL_PRIVATE_FEATURE_QUERY_FRAMEBUFFER_BITS,
+  COGL_PRIVATE_FEATURE_BUILTIN_POINT_SIZE_UNIFORM,
+  COGL_PRIVATE_FEATURE_QUERY_TEXTURE_PARAMETERS,
+  COGL_PRIVATE_FEATURE_ALPHA_TEXTURES,
+  COGL_PRIVATE_FEATURE_TEXTURE_SWIZZLE,
+  COGL_PRIVATE_FEATURE_TEXTURE_MAX_LEVEL,
+  COGL_PRIVATE_FEATURE_ARBFP,
+  COGL_PRIVATE_FEATURE_OES_EGL_SYNC,
   /* If this is set then the winsys is responsible for queueing dirty
    * events. Otherwise a dirty event will be queued when the onscreen
    * is first allocated or when it is shown or resized */
-  COGL_PRIVATE_FEATURE_DIRTY_EVENTS = 1L<<24,
-  COGL_PRIVATE_FEATURE_ENABLE_PROGRAM_POINT_SIZE = 1L<<25,
+  COGL_PRIVATE_FEATURE_DIRTY_EVENTS,
+  COGL_PRIVATE_FEATURE_ENABLE_PROGRAM_POINT_SIZE,
   /* These features let us avoid conditioning code based on the exact
    * driver being used and instead check for broad opengl feature
    * sets that can be shared by several GL apis */
-  COGL_PRIVATE_FEATURE_ANY_GL = 1L<<26,
-  COGL_PRIVATE_FEATURE_GL_FIXED = 1L<<27,
-  COGL_PRIVATE_FEATURE_GL_PROGRAMMABLE = 1L<<28,
-  COGL_PRIVATE_FEATURE_GL_EMBEDDED = 1L<<29,
-  COGL_PRIVATE_FEATURE_GL_WEB = 1L<<30
-} CoglPrivateFeatureFlags;
+  COGL_PRIVATE_FEATURE_ANY_GL,
+  COGL_PRIVATE_FEATURE_GL_FIXED,
+  COGL_PRIVATE_FEATURE_GL_PROGRAMMABLE,
+  COGL_PRIVATE_FEATURE_GL_EMBEDDED,
+  COGL_PRIVATE_FEATURE_GL_WEB,
+
+  COGL_N_PRIVATE_FEATURES
+} CoglPrivateFeature;
 
 /* Sometimes when evaluating pipelines, either during comparisons or
  * if calculating a hash value we need to tweak the evaluation
@@ -102,6 +104,9 @@ _cogl_push_source (CoglPipeline *pipeline, CoglBool enable_legacy);
 CoglBool
 _cogl_get_enable_legacy_state (void);
 
+#define _cogl_has_private_feature(ctx, feature) \
+  COGL_FLAGS_GET ((ctx)->private_features, (feature))
+
 /*
  * _cogl_pixel_format_get_bytes_per_pixel:
  * @format: a #CoglPixelFormat
