@@ -34,6 +34,7 @@
 struct _CoglTexture2DSliced
 {
   CoglTexture _parent;
+
   GArray *slice_x_spans;
   GArray *slice_y_spans;
   GArray *slice_textures;
@@ -49,14 +50,11 @@ _cogl_texture_2d_sliced_new_from_foreign (CoglContext *context,
                                           int height,
                                           int x_pot_waste,
                                           int y_pot_waste,
-                                          CoglPixelFormat  format,
-                                          CoglError **error);
+                                          CoglPixelFormat format);
 
 CoglTexture2DSliced *
 _cogl_texture_2d_sliced_new_from_bitmap (CoglBitmap *bmp,
                                          int max_waste,
-                                         CoglPixelFormat internal_format,
-                                         CoglBool can_convert_in_place,
-                                         CoglError **error);
+                                         CoglBool can_convert_in_place);
 
 #endif /* __COGL_TEXTURE_2D_SLICED_PRIVATE_H */
