@@ -72,7 +72,7 @@ typedef struct
 
   /* This builtin is only necessary if the following private feature
    * is not implemented in the driver */
-  CoglPrivateFeatureFlags feature_replacement;
+  CoglPrivateFeature feature_replacement;
 } BuiltinUniformData;
 
 static BuiltinUniformData builtin_uniforms[] =
@@ -139,6 +139,8 @@ typedef struct
   int flushed_flip_state;
 
   UnitState *unit_state;
+
+  CoglPipelineCacheEntry *cache_entry;
 } CoglPipelineProgramState;
 
 static CoglUserDataKey program_state_key;
@@ -228,7 +230,8 @@ clear_flushed_matrix_stacks (CoglPipelineProgramState *program_state)
 }
 
 static CoglPipelineProgramState *
-program_state_new (int n_layers)
+program_state_new (int n_layers,
+                   CoglPipelineCacheEntry *cache_entry)
 {
   CoglPipelineProgramState *program_state;
 
@@ -238,6 +241,7 @@ program_state_new (int n_layers)
   program_state->unit_state = g_new (UnitState, n_layers);
   program_state->uniform_locations = NULL;
   program_state->attribute_locations = NULL;
+  program_state->cache_entry = cache_entry;
   _cogl_matrix_entry_cache_init (&program_state->modelview_cache);
   _cogl_matrix_entry_cache_init (&program_state->projection_cache);
 
@@ -259,6 +263,10 @@ destroy_program_state (void *user_data,
   if (program_state->last_used_for_pipeline == instance)
     program_state->last_used_for_pipeline = NULL;
 
+  if (program_state->cache_entry &&
+      program_state->cache_entry->pipeline != instance)
+    program_state->cache_entry->usage_count--;
+
   if (--program_state->ref_count == 0)
     {
       clear_attribute_cache (program_state);
@@ -282,6 +290,17 @@ static void
 set_program_state (CoglPipeline *pipeline,
                   CoglPipelineProgramState *program_state)
 {
+  if (program_state)
+    {
+      program_state->ref_count++;
+
+      /* If we're not setting the state on the template pipeline then
+       * mark it as a usage of the pipeline cache entry */
+      if (program_state->cache_entry &&
+          program_state->cache_entry->pipeline != pipeline)
+        program_state->cache_entry->usage_count++;
+    }
+
   _cogl_object_set_user_data (COGL_OBJECT (pipeline),
                               &program_state_key,
                               program_state,
@@ -441,8 +460,8 @@ update_builtin_uniforms (CoglContext *context,
     return;
 
   for (i = 0; i < G_N_ELEMENTS (builtin_uniforms); i++)
-    if ((context->private_feature_flags &
-         builtin_uniforms[i].feature_replacement) == 0 &&
+    if (!_cogl_has_private_feature (context,
+                                    builtin_uniforms[i].feature_replacement) &&
         (program_state->dirty_builtin_uniforms & (1 << i)) &&
         program_state->builtin_uniform_locations[i] != -1)
       builtin_uniforms[i].update_func (pipeline,
@@ -641,7 +660,7 @@ _cogl_pipeline_progend_glsl_end (CoglPipeline *pipeline,
   CoglBool program_changed = FALSE;
   UpdateUniformsState state;
   CoglProgram *user_program;
-  CoglPipeline *template_pipeline = NULL;
+  CoglPipelineCacheEntry *cache_entry = NULL;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -673,33 +692,30 @@ _cogl_pipeline_progend_glsl_end (CoglPipeline *pipeline,
           if (G_LIKELY (!(COGL_DEBUG_ENABLED
                           (COGL_DEBUG_DISABLE_PROGRAM_CACHES))))
             {
-              template_pipeline =
+              cache_entry =
                 _cogl_pipeline_cache_get_combined_template (ctx->pipeline_cache,
                                                             authority);
 
-              program_state = get_program_state (template_pipeline);
+              program_state = get_program_state (cache_entry->pipeline);
             }
 
           if (program_state)
             program_state->ref_count++;
           else
             program_state
-              = program_state_new (cogl_pipeline_get_n_layers (authority));
+              = program_state_new (cogl_pipeline_get_n_layers (authority),
+                                   cache_entry);
 
           set_program_state (authority, program_state);
 
-          if (template_pipeline)
-            {
-              program_state->ref_count++;
-              set_program_state (template_pipeline, program_state);
-            }
+          program_state->ref_count--;
+
+          if (cache_entry)
+            set_program_state (cache_entry->pipeline, program_state);
         }
 
       if (authority != pipeline)
-        {
-          program_state->ref_count++;
-          set_program_state (pipeline, program_state);
-        }
+        set_program_state (pipeline, program_state);
     }
 
   /* If the program has changed since the last link then we do
@@ -790,8 +806,8 @@ _cogl_pipeline_progend_glsl_end (CoglPipeline *pipeline,
       clear_flushed_matrix_stacks (program_state);
 
       for (i = 0; i < G_N_ELEMENTS (builtin_uniforms); i++)
-        if ((ctx->private_feature_flags &
-             builtin_uniforms[i].feature_replacement) == 0)
+        if (!_cogl_has_private_feature
+            (ctx, builtin_uniforms[i].feature_replacement))
           GE_RET( program_state->builtin_uniform_locations[i], ctx,
                   glGetUniformLocation (gl_program,
                                         builtin_uniforms[i].uniform_name) );
@@ -847,8 +863,8 @@ _cogl_pipeline_progend_glsl_pre_change_notify (CoglPipeline *pipeline,
       int i;
 
       for (i = 0; i < G_N_ELEMENTS (builtin_uniforms); i++)
-        if ((ctx->private_feature_flags &
-             builtin_uniforms[i].feature_replacement) == 0 &&
+        if (!_cogl_has_private_feature
+            (ctx, builtin_uniforms[i].feature_replacement) &&
             (change & builtin_uniforms[i].change))
           {
             CoglPipelineProgramState *program_state
